﻿using UniversityContracts.BindingModels;
using UniversityContracts.SearchModels;
using UniversityContracts.StoragesContracts;
using UniversityContracts.ViewModels;
using UniversityDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;

namespace UniversityDatabaseImplement.Implements
{
    internal class DisciplineStorage : IDisciplineStorage
    {
        public List<DisciplineViewModel> GetFullList()
        {
            using var context = new UniversityDatabase();
            return context.Disciplines
                .Include(x => x.Statements)
                .Select(x => x.GetViewModel).ToList();
        }
        public List<DisciplineViewModel> GetFilteredList(DisciplineSearchModel model)
        {
            using var context = new UniversityDatabase();
            return context.Disciplines
                .Include(x => x.Statements)
                .Where(x => (
                        (!model.Id.HasValue || x.Id == model.Id) &&
                        (string.IsNullOrEmpty(model.Name) || x.Name.Contains(model.Name))
                    )
                )
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public DisciplineViewModel? GetElement(DisciplineSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new UniversityDatabase();
            return context.Disciplines
                .Include(x => x.Statements)
                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.Name) && x.Name == model.Name) ||
                                 (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }
        public DisciplineViewModel? Insert(DisciplineBindingModel model)
        {
            var newDiscipline = Discipline.Create(model);
            if (newDiscipline == null)
            {
                return null;
            }
            using var context = new UniversityDatabase();
            context.Disciplines.Add(newDiscipline);
            context.SaveChanges();
            return newDiscipline.GetViewModel;
        }
        public DisciplineViewModel? Update(DisciplineBindingModel model)
        {
            using var context = new UniversityDatabase();
            var discipline = context.Disciplines.FirstOrDefault(x => x.Id == model.Id);
            if (discipline == null)
            {
                return null;
            }
            discipline.Update(model);
            context.SaveChanges();
            return discipline.GetViewModel;
        }
        public DisciplineViewModel? Delete(DisciplineBindingModel model)
        {
            using var context = new UniversityDatabase();
            var element = context.Disciplines.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Disciplines.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
