﻿using UniversityContracts.BindingModels;
using UniversityContracts.ViewModels;
using UniversityDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace UniversityDatabaseImplement.Models
{
    public class Discipline : IDisciplineModel
    {
        public int Id { get; set; }
        [Required]
        public string Name { get; set; } = string.Empty;
        [Required]
        public string Department { get; set; } = string.Empty;
        [ForeignKey("DisciplineId")]
        public virtual List<Statement> Statements { get; set; } = new();
        [ForeignKey("DisciplineId")]
        public virtual List<ReportTypeDiscipline> ReportTypeDisciplines { get; set; } = new();
        private Dictionary<int, IStatementModel>? _disciplineStatements;
        [NotMapped]
        public Dictionary<int, IStatementModel> DisciplineStatements
        {
            get
            {
                if (_disciplineStatements == null)
                {
                    _disciplineStatements = Statements.ToDictionary(
                        x => x.Id, x => x as IStatementModel);
                }

                return _disciplineStatements;
            }
        }

        public static Discipline Create(DisciplineBindingModel model)
        {
            return new Discipline
            {
                Id = model.Id,
                Name = model.Name,
                Department = model.Department
            };
        }

        public void Update(DisciplineBindingModel model)
        {
            if (model == null) return;

            Id = model.Id;
            Name = model.Name;
            Department = model.Department;
        }

        public DisciplineViewModel GetViewModel => new()
        {
            Id = Id,
            Name = Name,
            Department = Department
        };
    }
}
