﻿using UniversityContracts.BindingModels;
using UniversityContracts.BuisnessLogicContracts;
using UniversityContracts.SearchModels;
using UniversityContracts.StoragesContracts;
using UniversityContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace UniversityBuisnessLogic.BuisnessLogic
{
    public class ReportLogic : IReportLogic
    {
        ILogger _logger;
        IDisciplineStorage _disciplineStorage;
        IStatementStorage _statementStorage;
        IExaminationResultStorage _examinationResultStorage;

        public ReportLogic(ILogger<ReportLogic> logger, IDisciplineStorage disciplineStorage,
                           IStatementStorage statementStorage, IExaminationResultStorage examinationResultStorage)
        {
            _logger = logger;
            _disciplineStorage = disciplineStorage;
            _statementStorage = statementStorage;
            _examinationResultStorage = examinationResultStorage;
        }

        public List<ReportStudentsViewModel> GetStudens(ReportBindingModel model)
        {
            if (model == null) return new();

            var results = _examinationResultStorage.GetFilteredList(new ExaminationResultSearchModel
            {
                From = model.From,
                To = model.To,
            });

            var statements = _statementStorage.GetFilteredList(new StatementSearchModel
            {
                From = model.From,
                To = model.To,
            });

            List<ReportStudentsViewModel> list = new();

            foreach (var result in results)
            {
                foreach(var student in result.Students.Values)
                {
                    foreach(var statement in statements)
                    {
                        if (!student.Statements.ContainsKey(statement.Id)) continue;

                        list.Add(new ReportStudentsViewModel
                        {
                            StudentName = student.Name,
                            ExaminationForm = result.ExaminationForm,
                            mark = result.Mark,
                            ExaminationResultDate = result.Date,
                            HoursCount = statement.HoursCount,
                            StatementDate = statement.Date
                        });
                    }
                }
            }

            return list;
        }

        public List<ReportStudentDisciplineViewModel> GetStudentDiscipline(ReportBindingModel model)
        {
            if (model == null || model.Students == null) return new();

            var disciplines = _disciplineStorage.GetFullList();
            List<ReportStudentDisciplineViewModel> result = new();

            foreach(var student in model.Students)
            {
                var record = new ReportStudentDisciplineViewModel
                {
                    StudentName = student.Name,
                };

                foreach(var discipline in disciplines)
                {
                    if(student.Statements.ContainsKey(discipline.Id))
                    {
                        record.Disciplines.Add(discipline.Name);
                    }
                }

                result.Add(record);
            }

            return result;
        }

        public void SaveStudentsToExcel()
        {
            throw new NotImplementedException("Реализация в следующем этапе");
        }

        public void SaveStudentsToPdf()
        {
            throw new NotImplementedException("Реализация в следующем этапе");
        }

        public void SaveStudentsToWord()
        {
            throw new NotImplementedException("Реализация в следующем этапе");
        }
    }
}
